from datetime import timedelta
from uuid import uuid4

DEFAULT_EMR_STEPS = [
    {
        "HadoopJarStep": {
            "Args": ["aws", "s3", "cp", "s3://bigabid-emr-bootstrap/amit/pyspark_quick_setup.sh", "/home/hadoop/"],
            "Jar": "command-runner.jar",
            "Properties": []
        },
        "ActionOnFailure": "CANCEL_AND_WAIT",
        "Name": "copy pyspark setup file"
    },
    {
        "HadoopJarStep": {
            "Args": ["sudo", "bash", "/home/hadoop/pyspark_quick_setup.sh"],
            "Jar": "command-runner.jar",
            "Properties": []
        },
        "ActionOnFailure": "CANCEL_AND_WAIT",
        "Name": "execute pyspark setup file"
    }
]
 
DEFAULT_EMR_PM_STEPS = [
    {
        "HadoopJarStep": {
            "Args": ["aws", "s3", "cp", "s3://bigabid-predictive-matching/bootstraps/pyspark_quick_setup.sh", "/home/hadoop/"],
            "Jar": "command-runner.jar",
            "Properties": []
        },
        "ActionOnFailure": "CANCEL_AND_WAIT",
        "Name": "copy pyspark setup file"
    },
    {
        "HadoopJarStep": {
            "Args": ["sudo", "bash", "/home/hadoop/pyspark_quick_setup.sh"],
            "Jar": "command-runner.jar",
            "Properties": []
        },
        "ActionOnFailure": "CANCEL_AND_WAIT",
        "Name": "execute pyspark setup file"
    }
]

DEFAULT_EMR_SPARK_WITH_NOTEBOOK_STEPS = [{}]

DEFAULT_EMR_SPARK_WITH_NOTEBOOK_JOB_FLOW_OVERRIDES = {
    "Name": str(uuid4())[0: 8],
    "LogUri": "s3://bigabid-ml/logs/",
    "Applications": [
        {
            "Name": "Spark"
        },
        {
            "Name": "Ganglia"
        },
        {
            "Name": "Livy"
        },
        {
            "Name": "Hadoop"
        },
        {
            "Name": "JupyterEnterpriseGateway"
        },
        {
            "Name": "JupyterHub"
        }
    ],
    "AutoTerminationPolicy": {
        "IdleTimeout": timedelta(hours=3).seconds
    },
    "ReleaseLabel": "emr-6.5.0",
    "Instances": {
        "InstanceGroups": [],
        "InstanceFleets": [
            {
                "Name": "MASTER",
                "InstanceFleetType": "MASTER",
                "TargetOnDemandCapacity": 1,
                "TargetSpotCapacity": 0,
                "InstanceTypeConfigs": [
                    {
                        "InstanceType": "r5d.4xlarge",
                        "WeightedCapacity": 1,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": []
                        }
                    },
                    {
                        "InstanceType": "r5.4xlarge",
                        "WeightedCapacity": 1,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": [
                                {
                                    "VolumeSpecification": {
                                        "VolumeType": "gp2",
                                        "SizeInGB": 256
                                    },
                                    "VolumesPerInstance": 1
                                }
                            ],
                            "EbsOptimized": True
                        }
                    }
                ],
                "LaunchSpecifications": {}
            },
            {
                "Name": "CORE",
                "InstanceFleetType": "CORE",
                "TargetOnDemandCapacity": 0,
                "TargetSpotCapacity": 1,
                "InstanceTypeConfigs": [
                    {
                        "InstanceType": "r5.4xlarge",
                        "WeightedCapacity": 1,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": [
                                {
                                    "VolumeSpecification": {
                                        "VolumeType": "gp2",
                                        "SizeInGB": 256
                                    },
                                    "VolumesPerInstance": 1
                                }
                            ],
                            "EbsOptimized": True
                        }
                    },
                    {
                        "InstanceType": "r5.8xlarge",
                        "WeightedCapacity": 2,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": [
                                {
                                    "VolumeSpecification": {
                                        "VolumeType": "gp2",
                                        "SizeInGB": 512
                                    },
                                    "VolumesPerInstance": 1
                                }
                            ],
                            "EbsOptimized": True
                        }
                    },
                    {
                        "InstanceType": "r5d.12xlarge",
                        "WeightedCapacity": 3,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": []
                        }
                    },
                    {
                        "InstanceType": "i3en.12xlarge",
                        "WeightedCapacity": 3,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": []
                        }
                    }
                ],
                "LaunchSpecifications": {
                    "SpotSpecification": {
                        "TimeoutDurationMinutes": 60,
                        "TimeoutAction": "TERMINATE_CLUSTER"
                    }
                }
            },
            {
                "Name": "TASK_FLEET_BY_MANAGED_SCALING",
                "InstanceFleetType": "TASK",
                "TargetOnDemandCapacity": 0,
                "TargetSpotCapacity": 1,
                "InstanceTypeConfigs": [
                    {
                        "InstanceType": "r5.4xlarge",
                        "WeightedCapacity": 1,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": [
                                {
                                    "VolumeSpecification": {
                                        "VolumeType": "gp2",
                                        "SizeInGB": 256
                                    },
                                    "VolumesPerInstance": 1
                                }
                            ],
                            "EbsOptimized": True
                        }
                    },
                    {
                        "InstanceType": "r5.8xlarge",
                        "WeightedCapacity": 2,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": [
                                {
                                    "VolumeSpecification": {
                                        "VolumeType": "gp2",
                                        "SizeInGB": 512
                                    },
                                    "VolumesPerInstance": 1
                                }
                            ],
                            "EbsOptimized": True
                        }
                    },
                    {
                        "InstanceType": "r5d.12xlarge",
                        "WeightedCapacity": 3,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": []
                        }
                    },
                    {
                        "InstanceType": "i3en.12xlarge",
                        "WeightedCapacity": 3,
                        "BidPriceAsPercentageOfOnDemandPrice": 100,
                        "EbsConfiguration": {
                            "EbsBlockDeviceConfigs": []
                        }
                    }
                ],
                "LaunchSpecifications": {
                    "SpotSpecification": {
                        "TimeoutDurationMinutes": 60,
                        "TimeoutAction": "TERMINATE_CLUSTER"
                    }
                }
            }
        ],
        "Ec2KeyName": "ds-team",
        "Placement": {},
        "KeepJobFlowAliveWhenNoSteps": True,
        "TerminationProtected": True,
        "HadoopVersion": "3.2.1",
        "Ec2SubnetIds": [
            "subnet-00111e66939e718da",
            "subnet-03f648af64447a539",
            "subnet-4b711e3c",
            "subnet-7cee8c70",
            "subnet-874b34e2",
            "subnet-ae62eff7",
            "subnet-c73386ec",
            "subnet-e24c11d8"
        ],
        "EmrManagedMasterSecurityGroup": "sg-526e8e34",
        "EmrManagedSlaveSecurityGroup": "sg-536e8e35",
        "AdditionalMasterSecurityGroups": [],
        "AdditionalSlaveSecurityGroups": []
    },
    "BootstrapActions": [],
    "Configurations": [
        {
            "Classification": "spark",
            "Properties": {
                "MaximizeResourceAllocation": "true"
            }
        },
        {
            "Classification": "spark-defaults",
            "Properties": {
                "spark.executor.memory": "36g",
                "spark.kryoserializer.buffer.max": "1024m",
                "spark.driver.maxResultSize": "0",
                "spark.rdd.compress": "true",
                "spark.network.timeout": "600s",
                "spark.executor.cores": "5",
                "spark.checkpoint.compress": "true",
                "spark.hadoop.fs.s3.consistent.retryPolicyType": "exponential",
                "spark.hadoop.fs.s3.maxRetries": "30",
                "spark.python.profile": "true",
                "spark.serializer": "org.apache.spark.serializer.KryoSerializer",
                "spark.jars.repositories": "https://repos.spark-packages.org/",
                "spark.executor.extraJavaOptions": "-XX:+UseG1GC -XX:+UnlockDiagnosticVMOptions -XX:+G1SummarizeConcMark -XX:InitiatingHeapOccupancyPercent=35 -XX:OnOutOfMemoryError='kill -9 %p'",
                "spark.dynamicAllocation.enabled": "true",
                "spark.driver.extraJavaOptions": "-XX:+UseG1GC -XX:+UnlockDiagnosticVMOptions -XX:+G1SummarizeConcMark -XX:InitiatingHeapOccupancyPercent=35 -XX:OnOutOfMemoryError='kill -9 %p'"
            }
        },
        {
            "Classification": "spark-hive-site",
            "Properties": {
                "hive.metastore.client.factory.class": "com.amazonaws.glue.catalog.metastore.AWSGlueDataCatalogHiveClientFactory"
            }
        }
    ],
    "VisibleToAllUsers": True,
    "Tags": [
        {
            "Key": "owner",
            "Value": "Airflow"
        },
        {
            "Key": "team",
            "Value": "DE"
        },
        {
            "Key": "env",
            "Value": "Development"
        },
        {
            "Key": "component",
            "Value": "Jupyter - PySpark"
        }
    ],
    "EbsRootVolumeSize": 20,
    "StepConcurrencyLevel": 1,
    "ManagedScalingPolicy": {
        "ComputeLimits": {
            "UnitType": "InstanceFleetUnits",
            "MinimumCapacityUnits": 2,
            "MaximumCapacityUnits": 5,
            "MaximumOnDemandCapacityUnits": 1,
            "MaximumCoreCapacityUnits": 1
        }
    },
    "JobFlowRole": "EMR_EC2_DefaultRole",
    "ServiceRole": "EMR_DefaultRole"
}