import os,socket, boto3
from croniter import croniter
from airflow.utils.dates import days_ago
from datetime import datetime


dag_vars_queries= {
    "queries":[
        {
                "name":"test5min",
                "category": "random",
                "cron":"*/5 * * * *",
                "query":"""SELECT count(1)
                    FROM "upsolver_tests"."parquet_test_attribution_prod_level_1"
                    where request_unixtime between {{(1645912401+macros.random() * 500) | int }} and {{(1645922401+macros.random() * 500) | int}}"""
        }, {
                "name":"test10min",
                "category" : "random",
                "cron":"*/10 * * * *",
                "query":"""SELECT count(1)
                    FROM "upsolver_tests"."parquet_test_attribution_prod_level_1"
                    where request_unixtime between {{(1645890401+macros.random() * 1000) | int}} and {{(1645892001+macros.random() * 1000) | int}}"""
        }, {
                "name":"parquet_impressions_for_bi_hourly",
                "category" : "random",
                "cron":"0 * * * *",
                "query":"""SELECT count(1)
                            FROM live_funnel.parquet_impressions_for_bi
                            where FROM_UNIXTIME(bid_unixtime) > current_timestamp - INTERVAL '1' HOUR"""
        }
    ]
}


def publish_metric(client, name, value, cat, unit='None'):
    environment_name = os.getenv("AIRFLOW__CORE__ENVIRONMENT", 'development')
    value_number = float(value)
    hostname = socket.gethostname()
    ip_address = socket.gethostbyname(hostname)
    print('writing value', value_number, 'to metric', name)
    response = client.put_metric_data(
        Namespace='MWAA-Custom',
        MetricData=[
            {
                'MetricName': name,
                'Dimensions': [
                    {
                        'Name': 'Environment',
                        'Value': environment_name
                    },
                    {
                        'Name': 'Category',
                        'Value': cat
                    }
                ],
                'Timestamp': datetime.now(),
                'Value': value_number,
                'Unit': unit
            },
        ]
    )
    print(response)
    return response


def handle_result(task_id, metric_name, category, **kwargs):
    ti = kwargs['ti']
    res = ti.xcom_pull(key='return_value', task_ids=task_id)
    body = res['ResultSet']['Rows']
    final_res = None
    for i, res in enumerate(body[1:]):
        final_res = res['Data'][0]['VarCharValue']

    client = boto3.client('cloudwatch')
    publish_metric(client, name=metric_name, cat=category, value=final_res, unit='Count')
