from airflow.models import taskinstance
from airflow.utils.state import State
from airflow.utils.db import provide_session


@provide_session
def clear_task(task_id, session = None, adr = False, dag = None):
    taskinstance.clear_task_instances(tis = task_id,
                                      session = session,
                                      activate_dag_runs = adr,
                                      dag = dag)

def retry_upstream_task(context):
    tasks = context["dag_run"].get_task_instances()
    to_retry = context["params"].get("to_retry", [])
    mark_failed = context["params"].get("mark_failed", [])
    dag = context["dag"]
    tasks_to_retry = [ti for ti in tasks if ti.task_id in to_retry]
    task_to_mark = [ti for ti in tasks if ti.task_id in mark_failed]
    clear_task(task_to_mark, dag=dag)
    for task in tasks_to_retry:
        task.state = State.UP_FOR_RETRY
        execute(task)

@provide_session
def execute(task, session=None):
    session.merge(task)
