from airflow.models import taskinstance
from airflow.utils.state import State
from airflow.utils.db import provide_session


@provide_session
def clear_task(task_id, session = None, adr = False, dag = None):
    taskinstance.clear_task_instances(tis = task_id,
                                      session = session,
                                      activate_dag_runs = adr,
                                      dag = dag)

def retry_upstream_task(context):
    tasks = context["dag_run"].get_task_instances()
    dag = context["dag"]
    to_retry = context["params"].get("to_retry", [])

    tasks_to_retry = [ti for ti in tasks if ti.task_id in to_retry]

    clear_task(tasks_to_retry, dag=dag)


@provide_session
def execute(task, session=None):
    session.merge(task)

def mark_upstream_failed(context) -> None:
    tasks = context["dag_run"].get_task_instances()
    mark_failed = context["params"].get("mark_failed", [])

    task_to_mark = [ti for ti in tasks if ti.task_id in mark_failed]

    for task in task_to_mark:
        task.state = State.FAILED
        execute(task)