"""
NOTE THIS IS UNTESTED AS IT WAS NOT REQUIRED.
See:
    - https://github.com/apache/airflow/blob/master/airflow/contrib/hooks/aws_hook.py
    - https://github.com/apache/airflow/blob/master/airflow/contrib/operators/ecs_operator.py
    - https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.ServiceResource.create_instances
    - https://stackabuse.com/automating-aws-ec2-management-with-python-and-boto3/
    - https://devops.stackexchange.com/questions/3264/how-to-run-a-script-or-a-command-on-ec2-instance-via-aws-cli?rq=1
We have granted AmazonSSMFullAccess and EMRAllActions to IAM user CloudComposer.pipeline.platform
"""

from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook
from airflow.exceptions import AirflowException
from airflow.models import BaseOperator
from airflow.plugins_manager import AirflowPlugin
from airflow.utils.decorators import apply_defaults
import base64
import time
from datetime import datetime, timedelta

DEFAULT_INSTANCE_CONFIG = {
    'BlockDeviceMappings': [
        {
            'DeviceName': '/dev/xvda',
            'Ebs': {
                'VolumeSize': 123,
                'VolumeType': 'gp2',
                'DeleteOnTermination': True
            }
        },
    ],
    'ImageId': 'ami-04681a1dbd79675a5',
    'InstanceType': 'm1.medium',
    'MaxCount': 1,
    'MinCount': 1,
    'Monitoring': {
        'Enabled': False
    }
}

class CreateEC2Operator(BaseOperator):
    ui_color = '#f0ede4'
    client = None
    template_fields = ('instance_config', 'user_data')

    @apply_defaults
    def __init__(self,
                 instance_config,
                 timedelta_days: int = 0,
                 aws_conn_id=None,
                 region_name=None,
                 user_data=None,
                 instance_name=None,
                 iam_role_arn=None,
                 **kwargs):
        super(CreateEC2Operator, self).__init__(**kwargs)

        self.aws_conn_id = aws_conn_id
        self.region_name = region_name
        self.instance_config = instance_config
        self.timedelta_days = timedelta_days
        self.user_data = user_data
        self.instance_name = instance_name
        self.iam_role_arn = iam_role_arn
        self.hook = self.get_hook()
        self.iam_instance_profile = {}

    def execute(self, context):
        self.log.info('Starting EC2 instance')

        # conn = self.hook.get_connection(self.aws_conn_id)
        self.client = self.hook.get_client_type('ec2', region_name=self.region_name)
        # self.client.describe_spot_instance_requests
        if self.iam_role_arn:
            self.iam_instance_profile['Arn'] = self.iam_role_arn

        if self.user_data:
            user_data = self.user_data
            execution_date = context.get("ds")
            run_date = datetime.fromisoformat(execution_date) - timedelta(days=self.timedelta_days)
            year = run_date.strftime("%Y")
            month = run_date.strftime("%m")
            day = run_date.strftime("%d")
            run_date = f'{year}/{month}/{day}'
            user_data = user_data.format(run_date=run_date)

            self.instance_config['UserData'] = user_data

        self.log.info(self.instance_config)

        response = self.client.request_spot_instances(
            InstanceCount=1,
            LaunchSpecification={
                'SecurityGroupIds': [
                    'sg-c6b7bfa0'
                ],
                'BlockDeviceMappings': [
                    {
                        'DeviceName': '/dev/sda1',
                        'Ebs': {
                            'DeleteOnTermination': True,
                            'VolumeSize': 40,
                            'VolumeType': 'gp2'
                        }
                    }
                ],
                'IamInstanceProfile': self.iam_instance_profile,
                'UserData': base64.b64encode(self.instance_config['UserData'].encode("ascii")).decode('ascii'),
                'Monitoring': {
                    'Enabled': False
                },
                'ImageId': 'ami-034a759569cd204a6',
                'InstanceType': 'm4.xlarge',
                'KeyName': "ds-team",
                
            },
            SpotPrice='1.0',
        )

        spot_instance_request_ids = []
        for instance in response['SpotInstanceRequests']:
            spot_instance_request_ids.append(instance['SpotInstanceRequestId'])

        tries_limit = 8
        while True:
            tries_limit -= 1
            time.sleep(1)
            describe_spot_response = self.client.describe_spot_instance_requests(
                SpotInstanceRequestIds=spot_instance_request_ids,
            )
            if tries_limit <= 0 or describe_spot_response['SpotInstanceRequests'][0]['Status']['Code'] == 'fulfilled':
                break

        if describe_spot_response['SpotInstanceRequests'][0]['Status']['Code'] == 'fulfilled':
            instance_ids = []
            for instance in describe_spot_response['SpotInstanceRequests']:
                instance_ids.append(instance['InstanceId'])

            tag_response = self.client.create_tags(
                Resources=instance_ids,
                Tags=[{
                    'Key': 'Name',
                    'Value': self.instance_name
                }]
            )

        # TODO: Return instance id (xcom)
        # TODO: Give instance a name
        # TODO: Give Airflow IAM user SSM access perms

    def get_hook(self):
        return AwsBaseHook(aws_conn_id=self.aws_conn_id, client_type="ec2")