from airflow.models.baseoperator import BaseOperator
from airflow.utils.decorators import apply_defaults
from airflow.contrib.hooks.aws_lambda_hook import AwsLambdaHook
from botocore.client import Config
import json


class RunAWSLambdaOperator(BaseOperator):

    @apply_defaults
    def __init__(
            self,
            function_name: str,
            payload: object = {},
            aws_conn_id=None,
            *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.function_name = function_name
        self.payload = payload
        self.aws_conn_id = aws_conn_id
        self.hook = self.get_hook()

    def execute(self, context):
        self.payload['execution_date'] = context.get("ds")
        payload = json.dumps(self.payload)
        response = self.hook.invoke_lambda(payload=payload)
        self.log.info(f'Response: {response}')
        if response['StatusCode'] == 200:
            bytes_value = response['Payload'].read()
            # Decode UTF-8 bytes to Unicode, and convert single quotes
            # to double quotes to make it valid JSON
            payload_json = bytes_value.decode('utf8').replace("'", '"')
            # Load the JSON to a Python list
            lambda_res = json.loads(payload_json)
            if lambda_res['statusCode'] != 200:
                raise ValueError(lambda_res['body'])
            return payload_json
        else:
            raise ValueError(json.dumps(response))

    def get_hook(self):
        return AwsLambdaHook(self.function_name,
                             region_name='us-east-1',
                             qualifier='$LATEST',
                             invocation_type='RequestResponse',
                             config=Config(
                                 read_timeout=900,
                                 connect_timeout=900,
                                 max_pool_connections=1,
                                 retries=dict(max_attempts=0)),
                             aws_conn_id=self.aws_conn_id)
