from airflow.models.baseoperator import BaseOperator
from airflow.utils.decorators import apply_defaults
from airflow.contrib.hooks.aws_lambda_hook import AwsLambdaHook
import json


class UpsolverClusterManagerOperator(BaseOperator):

    @apply_defaults
    def __init__(
            self,
            cluster_id: str,
            action: str,
            aws_conn_id=None,
            *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.cluster_id = cluster_id
        self.action = action
        self.aws_conn_id = aws_conn_id
        self.hook = self.get_hook()

    def execute(self, context):
        payload = json.dumps({'cluster_id': self.cluster_id, 'action': self.action})
        response = self.hook.invoke_lambda(payload=payload)
        if response['StatusCode'] == 200:
            return 'Success'
        else:
            raise ValueError(json.dumps(response))

    def get_hook(self):
        return AwsLambdaHook('UpsolverClusterManagerSta-UpsolverClusterManagerSt-1OHSMS8FCSHDS',
                             region_name='us-east-1',
                             qualifier='$LATEST',
                             invocation_type='RequestResponse',
                             config=None,
                             aws_conn_id=self.aws_conn_id)
