from urllib.parse import urlparse
from datetime import datetime, timedelta
from airflow.exceptions import AirflowException
from airflow.sensors.base_sensor_operator import BaseSensorOperator
from airflow.utils.decorators import apply_defaults
from airflow.hooks.S3_hook import S3Hook
from datetime import date


class S3IfFileUpdatedToday(BaseSensorOperator, S3Hook):

    @apply_defaults
    def __init__(self,
                 bucket_key,
                 bucket_name=None,
                 timedelta_days=0,
                 aws_conn_id='aws_default',
                 verify=None,
                 *args,
                 **kwargs):
        super(S3IfFileUpdatedToday, self).__init__(*args, **kwargs)
        # Parse
        if bucket_name is None:
            parsed_url = urlparse(bucket_key)
            if parsed_url.netloc == '':
                raise AirflowException('Please provide a bucket_name')
            else:
                bucket_name = parsed_url.netloc
                if parsed_url.path[0] == '/':
                    bucket_key = parsed_url.path[1:]
                else:
                    bucket_key = parsed_url.path
        self.bucket_name = bucket_name
        self.bucket_key = bucket_key
        self.timedelta_days = timedelta_days
        self.aws_conn_id = aws_conn_id
        self.verify = verify

    def poke(self, context):

        paginator = self.get_conn().get_paginator('list_objects_v2')
        response = paginator.paginate(Bucket=self.bucket_name,
                                      Prefix=self.bucket_key)

        date_today = (datetime.now()).strftime("%Y-%m-%d")

        res_list_keys = []
        for page in response:
            if 'Contents' in page:
                for k in page['Contents']:
                    if (k["LastModified"]).strftime("%Y-%m-%d") == date_today:
                        res_list_keys.append(k['Key'])

        self.log.info(
            f'Number of items in s3://{self.bucket_name}/{self.bucket_key} updated today is {len(res_list_keys)}')
        if len(res_list_keys) >= 1:
            return True
        return False

    def set_date_to_bucket_key(self, date_today):
        return f'{self.bucket_key}/date={date_today}/'
