from airflow.utils.decorators import apply_defaults
from airflow.sensors.s3_key_sensor import S3KeySensor
from datetime import datetime, timedelta


class S3KeyDatePartitionSensor(S3KeySensor):

    @apply_defaults
    def __init__(self,
                 timedelta_days: int = 0,
                 *args,
                 **kwargs):
        super(S3KeyDatePartitionSensor, self).__init__(*args, **kwargs)
        self.timedelta_days = timedelta_days

    def poke(self, context):
        execution_date = context.get("ds")
        self.bucket_key = self.set_date_in_path(execution_date)
        return super().poke(context)

    def set_date_in_path(self, execution_date):
        date = datetime.fromisoformat(execution_date) - timedelta(days=self.timedelta_days)
        year = date.strftime("%Y")
        month = date.strftime("%m")
        day = date.strftime("%d")
        return self.bucket_key.format(year=year, month=month, day=day)
