from urllib.parse import urlparse
from datetime import datetime, timedelta
from airflow.exceptions import AirflowException
from airflow.sensors.base_sensor_operator import BaseSensorOperator
from airflow.utils.decorators import apply_defaults
from airflow.hooks.S3_hook import S3Hook


class S3ListKeysSensor(BaseSensorOperator):

    @apply_defaults
    def __init__(self,
                 bucket_key,
                 num_of_items,
                 bucket_name=None,
                 timedelta_days=0,
                 aws_conn_id='aws_default',
                 verify=None,
                 *args,
                 **kwargs):
        super(S3ListKeysSensor, self).__init__(*args, **kwargs)
        # Parse
        if bucket_name is None:
            parsed_url = urlparse(bucket_key)
            if parsed_url.netloc == '':
                raise AirflowException('Please provide a bucket_name')
            else:
                bucket_name = parsed_url.netloc
                if parsed_url.path[0] == '/':
                    bucket_key = parsed_url.path[1:]
                else:
                    bucket_key = parsed_url.path
        self.bucket_name = bucket_name
        self.num_of_items = num_of_items
        self.bucket_key = bucket_key
        self.timedelta_days = timedelta_days
        self.aws_conn_id = aws_conn_id
        self.verify = verify

    def poke(self, context):
        hook = S3Hook(aws_conn_id=self.aws_conn_id, verify=self.verify)
        execution_date = context.get("ds")
        self.bucket_key = self.set_date_in_path(execution_date)
        self.log.info('Poking for key : s3://%s/%s', self.bucket_name, self.bucket_key)
        res_list_keys = hook.list_keys(self.bucket_name, self.bucket_key)
        if len(res_list_keys) > 0 and '_SUCCESS' in res_list_keys[0]:
            res_list_keys.pop(0)

        self.log.info(f'Number of items in s3://{self.bucket_name}/{self.bucket_key} is {len(res_list_keys)}')
        if len(res_list_keys) == self.num_of_items:
            return True
        return False

    def set_date_in_path(self, execution_date):
        date = datetime.fromisoformat(execution_date) - timedelta(days=self.timedelta_days)
        year = date.strftime("%Y")
        month = date.strftime("%m")
        day = date.strftime("%d")
        return self.bucket_key.format(year=year, month=month, day=day)
